#open "unix";;
#open "misc";;
#open "instruct";;
#open "config";;
#open "opcodes";;
#open "buffcode";;
#open "labels";;
#open "symtable";;
#open "emit_code";;
#open "emit_data";;

(* Production du code pour un fichier objet *)

let link_object outchan name =
  let truename = find_in_path (name ^ ".zo") !modules__path_interfaces in
  let intchan = open_in truename in
    try
      object_name := name;
      while true do
        init_out_code();
        let (init, body) =
         try(input_value intchan : zam_instruction list * zam_instruction list)
         with failure x -> raise (failure (x ^ " (file " ^ truename ^ ")"))
        in
        reset_label_table();
        emit init;
        emit [Kbranch 0];
        emit body;
        align04();
        emit [Klabel 0];
        output outchan !out_buffer 0 !out_pos;
        abs_out_pos := !out_pos + !abs_out_pos
      done
    with End_of_file ->
      close_in intchan
;;

let write_symbols = ref false;;

(* Construction d'un executable bytecode *)

let link module_list exec_name =
  reset_linker_tables();
  reset_c_primitives();
  abs_out_pos := 0;
  function_beginnings := [];
  let outchan =
    open_out_gen (mode_rdwr + mode_trunc + mode_creat)
                 (rights_everything * rights_everybody) exec_name in
  try
    output_string outchan path_runtime;
    let pos0 = pos_out outchan in
    let pos1 = pos0 + 16 in
(* On reserve de la place pour l'en-tete *)
    seek_out outchan pos1;
(* Le bytecode *)
    do_list (link_object outchan) module_list;
    output_char outchan (char_of_int STOP);
(* Les donnees initialisees *)
    let pos2 = pos_out outchan in
    emit_data_to_file outchan;
(* Les tables du linker *)
    let pos3 = pos_out outchan in
    if !write_symbols then save_linker_tables outchan;
(* Les infos de debug (partielles) *)
    let pos4 = pos_out outchan in
(*  output_value outchan !function_beginnings; *)
(* On met a jour l'en-tete avec les tailles *)
    seek_out outchan pos0;
    output_int outchan (pos2 - pos1);
    output_int outchan (pos3 - pos2);
    output_int outchan (pos4 - pos3);
    output_int outchan 0;
    close_out outchan;
    output_c_primitives()
  with x ->
    unlink exec_name;
    close_out outchan;
    raise x
;;

